//
//  GameObject.m
//  MonsterWars
//
//  Created by Ray Wenderlich on 10/24/12.
//  Copyright (c) 2012 Razeware LLC. All rights reserved.
//

#import "GameObject.h"
#import "HelloWorldLayer.h"
#import "SimpleAudioEngine.h"
#import "Laser.h"
#import "Player.h"

@implementation GameObject

- (id)initWithSpriteFrameName:(NSString *)spriteFrameName layer:(HelloWorldLayer *)layer {
    if ((self = [super initWithSpriteFrameName:spriteFrameName])) {
        self.layer = layer;
        self.alive = YES;
        [self scheduleUpdateWithPriority:1];
    }
    return self;
}

- (void)draw {
    
    if (self.maxHp == 0) return;
    
    int sX = self.position.x - self.contentSize.width/2;
    int eX = self.position.x + self.contentSize.width/2;
    int actualY = self.position.y + self.contentSize.height/2;
    
    static int maxColor = 200;
    static int colorBuffer = 55;
    float percentage = ((float) self.curHp) / ((float) self.maxHp);
    int actualX = ((eX-sX) * percentage) + sX;
    int amtRed = ((1.0f-percentage)*maxColor)+colorBuffer;
    int amtGreen = (percentage*maxColor)+colorBuffer;
    
    glLineWidth(7);
    ccDrawColor4B(amtRed,amtGreen,0,255);
    ccDrawLine(ccp(sX, actualY), ccp(actualX, actualY));
    
}

- (void)update:(ccTime)delta {
    
    if (!self.alive) return;
    if (self.maxHp == 0) return;
        
    if (self.curHp <= 0) {
        
        [[SimpleAudioEngine sharedEngine] playEffect:@"boom.wav"];
        
        self.alive = FALSE;
        
        [self runAction:
         [CCSequence actions:
          [CCFadeOut actionWithDuration:0.5],
          [CCCallBlock actionWithBlock:^{
             [self.layer removeGameObject:self];
         }], nil]];
    }    
}

@end
