//
//  Monster.m
//  MonsterWars
//
//  Created by Ray Wenderlich on 10/24/12.
//  Copyright (c) 2012 Razeware LLC. All rights reserved.
//

#import "Monster.h"
#import "HelloWorldLayer.h"
#import "Player.h"
#import "SimpleAudioEngine.h"
#import "Laser.h"

@implementation Monster {
    BOOL _aoeDamageCaused;
}

- (void)updateMove:(ccTime)dt {
    
    if (self.maxAcceleration <= 0 || self.maxVelocity <= 0) return;
    
    BOOL hasTarget = FALSE;
    CGPoint moveTarget;
    GameObject * enemy = [self.layer closestEnemyToGameObject:self];
    float distance = ccpDistance(self.position, enemy.position);
    static float ATTACK_THRESHHOLD = 150;
    
    if (self.attacking || (enemy && distance < ATTACK_THRESHHOLD)) {
        if (!enemy) {
            enemy = [self.layer playerForTeam:[self.layer oppositeTeam:self.team]];
            distance = ccpDistance(self.position, enemy.position);
        }
        if (enemy) {
            hasTarget = YES;
            moveTarget = enemy.position;
            if (self.isRanged) {
                CGPoint vector = ccpNormalize(ccpSub(self.position, enemy.position));
                moveTarget = ccpAdd(enemy.position, ccpMult(vector, self.rangedRange));
                distance = ccpDistance(self.position, moveTarget);
            }
        }
    } else {
        Player * player = [self.layer playerForTeam:self.team];
        if (player) {
            hasTarget = YES;
            moveTarget = player.position;
        }
    }
    
    
    if (hasTarget) {
        CGPoint direction = ccpNormalize(ccpSub(moveTarget, self.position));
        CGPoint newPosition = ccpAdd(self.position, ccpMult(direction, self.maxVelocity * dt));
        CGSize winSize = [CCDirector sharedDirector].winSize;
        newPosition.x = MAX(MIN(newPosition.x, winSize.width), 0);
        newPosition.y = MAX(MIN(newPosition.y, winSize.height), 0);
        self.position = newPosition;
    }
}

- (void)checkCollision:(GameObject *)enemy {
    if (CGRectIntersectsRect(self.boundingBox, enemy.boundingBox)) {
        if (CACurrentMediaTime() - self.meleeLastDamageTime > self.meleeDamageRate) {
            
            [[SimpleAudioEngine sharedEngine] playEffect:self.meleeSound];
            
            if (self.meleeAoe) {
                _aoeDamageCaused = YES;
            } else {
                self.meleeLastDamageTime = CACurrentMediaTime();
            }
            enemy.curHp -= self.meleeDamage;
            if (enemy.curHp < 0) {
                enemy.curHp = 0;
            }
            if (self.meleeDestroySelf) {
                [self.layer removeGameObject:self];
            }
        }
    }
}

- (void)updateMelee:(ccTime)dt {
    
    if (!self.isMelee) return;
    
    NSArray * enemies = [self.layer enemiesOfTeam:self.team];
    _aoeDamageCaused = FALSE;
    for (GameObject * enemy in enemies) {
        [self checkCollision:enemy];
    }
    [self checkCollision:[self.layer playerForTeam:[self.layer oppositeTeam:self.team]]];
    
    // Special case for AOE damage - let it attack multiple things before we
    // reset the last damage time
    if (_aoeDamageCaused) {
        self.meleeLastDamageTime = CACurrentMediaTime();
    }
}

- (void)updateRanged:(ccTime)dt {
    
    if (!self.isRanged) return;
    
    GameObject * enemy = [self.layer closestEnemyToGameObject:self];
    if (!enemy) {
        enemy = [self.layer playerForTeam:[self.layer oppositeTeam:self.team]];
    }
    if (!enemy) return;
    
    float distance = ccpDistance(self.position, enemy.position);
    static float WIGGLE_ROOM = 5;
    if (ABS(distance) <= (self.rangedRange + WIGGLE_ROOM) && CACurrentMediaTime() - self.rangedLastDamageTime > self.rangedDamageRate) {
        
        [[SimpleAudioEngine sharedEngine] playEffect:self.rangedSound];
        
        self.rangedLastDamageTime = CACurrentMediaTime();
        
        Laser * laser = [self.layer createLaserForTeam:self.team];
        laser.position = self.position;
        laser.meleeDamage = self.rangedDamage;
        
        CGPoint direction = ccpNormalize(ccpSub(enemy.position, self.position));
        static float laserPointsPerSecond = 100;
        static float laserDistance = 1000;
        
        CGPoint target = ccpMult(direction, laserDistance);
        float duration = laserDistance / laserPointsPerSecond;
        
        laser.rotation = -1 * CC_RADIANS_TO_DEGREES(ccpToAngle(direction));
        
        [laser runAction:
         [CCSequence actions:
          [CCMoveBy actionWithDuration:duration position:target],
          [CCCallBlock actionWithBlock:^{
             [laser removeFromParentAndCleanup:YES];
         }], nil]];
    }
    
}

- (void)update:(ccTime)dt {
    
    [super update:dt];
    
    if (!self.alive) return;
    
    [self updateMelee:dt];
    [self updateRanged:dt];
    [self updateMove:dt];
    
}

@end
