//
//  HelloWorldLayer.m
//  MonsterWars
//
//  Created by Ray Wenderlich on 10/24/12.
//  Copyright Razeware LLC 2012. All rights reserved.
//


// Import the interfaces
#import "HelloWorldLayer.h"
#import "AppDelegate.h"
#import "SimpleAudioEngine.h"
#import "HumanPlayer.h"
#import "AIPlayer.h"
#import "QuirkMonster.h"
#import "ZapMonster.h"
#import "MunchMonster.h"
#import "Laser.h"

#pragma mark - HelloWorldLayer

// HelloWorldLayer implementation
@implementation HelloWorldLayer {
    CCLabelBMFont * _coin1Label;
    CCLabelBMFont * _coin2Label;
    CCLabelBMFont * _stateLabel;
    CCSpriteBatchNode * _batchNode;
    HumanPlayer * _humanPlayer;
    AIPlayer * _aiPlayer;
    NSMutableArray * _gameObjects;
    BOOL _gameOver;
}

// Helper class method that creates a Scene with the HelloWorldLayer as the only child.
+(CCScene *) scene
{
    // 'scene' is an autorelease object.
    CCScene *scene = [CCScene node];
    
    // 'layer' is an autorelease object.
    HelloWorldLayer *layer = [HelloWorldLayer node];
    
    // add layer as a child to scene
    [scene addChild: layer];
    
    // return the scene
    return scene;
}

- (void)basicSetup {
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    float MARGIN = 26 / CC_CONTENT_SCALE_FACTOR();
    
    // Create batch node
    [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"Sprites.plist"];
    _batchNode = [CCSpriteBatchNode batchNodeWithFile:@"Sprites.pvr.ccz"];
    [self addChild:_batchNode];
    
    // Sounds
    [[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"Latin_Industries.mp3"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"bigHit.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"boom.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"pew.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"pew2.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"smallHit.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"attack.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"defend.wav"];
    [[SimpleAudioEngine sharedEngine] preloadEffect:@"spawn.wav"];
    
    // Rest of UI
    
    CCSprite * background = [CCSprite spriteWithFile:@"background.png"];
    background.position = ccp(winSize.width/2, winSize.height/2);
    [self addChild:background z:-1];
    
    CCMenuItemSprite * quirkButton = [CCMenuItemSprite itemWithNormalSprite:[CCSprite spriteWithSpriteFrameName:@"button.png"] selectedSprite:[CCSprite spriteWithSpriteFrameName:@"button_sel.png"] target:self selector:@selector(quirkButtonTapped:)];
    CCMenuItemSprite * zapButton = [CCMenuItemSprite itemWithNormalSprite:[CCSprite spriteWithSpriteFrameName:@"button.png"] selectedSprite:[CCSprite spriteWithSpriteFrameName:@"button_sel.png"]target:self selector:@selector(zapButtonTapped:)];
    CCMenuItemSprite * munchButton = [CCMenuItemSprite itemWithNormalSprite:[CCSprite spriteWithSpriteFrameName:@"button.png"] selectedSprite:[CCSprite spriteWithSpriteFrameName:@"button_sel.png"]target:self selector:@selector(munchButtonTapped:)];
    
    zapButton.position = ccp(winSize.width/2, MARGIN + zapButton.contentSize.height/2);
    quirkButton.position = ccp(zapButton.position.x - zapButton.contentSize.width/2 - MARGIN - quirkButton.contentSize.width/2, MARGIN + zapButton.contentSize.height/2);
    munchButton.position = ccp(zapButton.position.x + zapButton.contentSize.width/2 + MARGIN + munchButton.contentSize.width/2, MARGIN + munchButton.contentSize.height/2);
    
    CCSprite * quirk = [CCSprite spriteWithSpriteFrameName:@"quirk1.png"];
    quirk.position = ccp(quirkButton.contentSize.width * 0.25, quirkButton.contentSize.height/2);
    [quirkButton addChild:quirk];
    
    CCSprite * zap = [CCSprite spriteWithSpriteFrameName:@"zap1.png"];
    zap.position = ccp(zapButton.contentSize.width * 0.25, zapButton.contentSize.height/2);
    [zapButton addChild:zap];
    
    CCSprite * munch = [CCSprite spriteWithSpriteFrameName:@"munch1.png"];
    munch.position = ccp(munchButton.contentSize.width * 0.25, munchButton.contentSize.height/2);
    [munchButton addChild:munch];
    
    CCLabelBMFont * quirkLabel = [CCLabelBMFont labelWithString:@"10" fntFile:@"Courier.fnt"];
    quirkLabel.position = ccp(quirkButton.contentSize.width * 0.75, MARGIN*2.1);
    [quirkButton addChild:quirkLabel];
    
    CCLabelBMFont * zapLabel = [CCLabelBMFont labelWithString:@"25" fntFile:@"Courier.fnt"];
    zapLabel.position = ccp(zapButton.contentSize.width * 0.75, MARGIN*2.1);
    [zapButton addChild:zapLabel];
    
    CCLabelBMFont * munchLabel = [CCLabelBMFont labelWithString:@"50" fntFile:@"Courier.fnt"];
    munchLabel.position = ccp(munchButton.contentSize.width * 0.75, MARGIN*2.1);
    [munchButton addChild:munchLabel];
    
    _stateLabel = [CCLabelBMFont labelWithString:@"Idle" fntFile:@"Courier.fnt"];
    _stateLabel.position = ccp(winSize.width/2, winSize.height * 0.25);
    [self addChild:_stateLabel];
    
    CCMenu * menu = [CCMenu menuWithItems:quirkButton, zapButton, munchButton, nil];
    menu.position = CGPointZero;
    [self addChild:menu];
    
    CCSprite * coin1 = [CCSprite spriteWithSpriteFrameName:@"coin.png"];
    coin1.position = ccp(MARGIN + coin1.contentSize.width/2, winSize.height - MARGIN - coin1.contentSize.height/2);
    [self addChild:coin1];
    
    CCSprite * coin2 = [CCSprite spriteWithSpriteFrameName:@"coin.png"];
    coin2.position = ccp(winSize.width - MARGIN - coin2.contentSize.width/2, winSize.height - MARGIN - coin2.contentSize.height/2);
    [self addChild:coin2];
        
    _coin1Label = [CCLabelBMFont labelWithString:@"10" fntFile:@"Courier.fnt" width:winSize.width * 0.25 alignment:kCCTextAlignmentLeft];
    _coin1Label.position = ccp(coin1.position.x + coin1.contentSize.width/2 + MARGIN/2 + _coin1Label.contentSize.width/2, winSize.height - MARGIN*1.6);
    [self addChild:_coin1Label];
    
    _coin2Label = [CCLabelBMFont labelWithString:@"10" fntFile:@"Courier.fnt" width:winSize.width * 0.25 alignment:kCCTextAlignmentRight];
    _coin2Label.position = ccp(coin2.position.x - coin1.contentSize.width/2 - MARGIN/2 - _coin2Label.contentSize.width/2, winSize.height - MARGIN*1.6);
    [self addChild:_coin2Label];
    
    self.isTouchEnabled = YES;
    [self scheduleUpdate];
}

- (void)addPlayers {
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    
    _humanPlayer = [[HumanPlayer alloc] initWithLayer:self];
    _humanPlayer.position = ccp(_humanPlayer.contentSize.width/2, winSize.height/2);
    //_humanPlayer.spawnPoint = ccp(_humanPlayer.position.x + _humanPlayer.contentSize.width/2, _humanPlayer.position.y);
    [_batchNode addChild:_humanPlayer];
    
    _aiPlayer = [[AIPlayer alloc] initWithLayer:self];
    _aiPlayer.position = ccp(winSize.width - _aiPlayer.contentSize.width/2, winSize.height/2);
    //_aiPlayer.spawnPoint = ccp(_aiPlayer.position.x - _aiPlayer.contentSize.width/2, _aiPlayer.position.y);
    [_batchNode addChild:_aiPlayer];
    
    //NSMutableArray * team1GameObjects = [NSMutableArray arrayWithObject:_humanPlayer];
    //NSMutableArray * team2GameObjects = [NSMutableArray arrayWithObject:_aiPlayer];
    //_gameObjects = [NSMutableArray arrayWithObjects:team1GameObjects, team2GameObjects, nil];
    _gameObjects = [NSMutableArray arrayWithObjects:[NSMutableArray array], [NSMutableArray array], nil];
    
}

// on "init" you need to initialize your instance
-(id) init
{
    // always call "super" init
    // Apple recommends to re-assign "self" with the "super's" return value
    if( (self=[super init]) ) {
                
        [self basicSetup];
        [self addPlayers];
             
    }
    return self;
}

- (void)showRestartMenu:(BOOL)won {
    
    if (_gameOver) return;
    _gameOver = YES;
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    
    NSString *message;
    if (won) {
        message = @"You win!";
    } else {
        message = @"You lose!";
    }
    
    CCLabelBMFont *label;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        label = [CCLabelBMFont labelWithString:message fntFile:@"Courier-hd.fnt"];
    } else {
        label = [CCLabelBMFont labelWithString:message fntFile:@"Courier.fnt"];
    }
    label.scale = 0.1;
    label.position = ccp(winSize.width/2, winSize.height * 0.6);
    [self addChild:label];
    
    CCLabelBMFont *restartLabel;
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        restartLabel = [CCLabelBMFont labelWithString:@"Restart" fntFile:@"Courier-hd.fnt"];
    } else {
        restartLabel = [CCLabelBMFont labelWithString:@"Restart" fntFile:@"Courier.fnt"];
    }
    
    CCMenuItemLabel *restartItem = [CCMenuItemLabel itemWithLabel:restartLabel target:self selector:@selector(restartTapped:)];
    restartItem.scale = 0.1;
    restartItem.position = ccp(winSize.width/2, winSize.height * 0.4);
    
    CCMenu *menu = [CCMenu menuWithItems:restartItem, nil];
    menu.position = CGPointZero;
    [self addChild:menu z:10];
    
    [restartItem runAction:[CCScaleTo actionWithDuration:0.5 scale:1.0]];
    [label runAction:[CCScaleTo actionWithDuration:0.5 scale:1.0]];
    
}

- (void)restartTapped:(id)sender {
    
    // Reload the current scene
    CCScene *scene = [HelloWorldLayer scene];
    [[CCDirector sharedDirector] replaceScene:[CCTransitionZoomFlipX transitionWithDuration:0.5 scene:scene]];
    
}

- (void)registerWithTouchDispatcher {
    [[[CCDirector sharedDirector] touchDispatcher] addTargetedDelegate:self priority:0 swallowsTouches:YES];
}

- (void)setPlayer:(Player *)player attacking:(BOOL)attacking {
    player.attacking = attacking;
    for (Monster * object in _gameObjects[player.team-1]) {
        object.attacking = attacking;
    }
}

- (BOOL)ccTouchBegan:(UITouch *)touch withEvent:(UIEvent *)event {
    
    CGPoint touchPoint = [self convertTouchToNodeSpace:touch];
    if (CGRectContainsPoint(_humanPlayer.boundingBox, touchPoint)) {
        BOOL attacking = !_humanPlayer.attacking;
        [self setPlayer:_humanPlayer attacking:attacking];
    }
    return YES;
}

- (void)spawnQuirkForPlayer:(Player *)player {
    if (player.coins < COST_QUIRK) return;
    player.coins -= COST_QUIRK;
    [[SimpleAudioEngine sharedEngine] playEffect:@"spawn.wav"];
    
    for (int i = 0; i < 2; ++i) {
        QuirkMonster * monster = [[QuirkMonster alloc] initWithTeam:player.team layer:self];
        
        CGSize winSize = [CCDirector sharedDirector].winSize;
        float randomOffset = CCRANDOM_X_Y(-winSize.height * 0.25, winSize.height * 0.25);
        //monster.position = ccp(player.spawnPoint.x + player.contentSize.width/2, player.spawnPoint.y + randomOffset);
        monster.position = ccp(player.position.x + player.contentSize.width/2, player.position.y + randomOffset);
        monster.attacking = player.attacking;
        
        [_batchNode addChild:monster];
        [_gameObjects[player.team-1] addObject:monster];
    }
}

- (void)spawnZapForPlayer:(Player *)player {
    if (player.coins < COST_ZAP) return;
    player.coins -= COST_ZAP;
    [[SimpleAudioEngine sharedEngine] playEffect:@"spawn.wav"];

    ZapMonster * monster = [[ZapMonster alloc] initWithTeam:player.team layer:self];
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    float randomOffset = CCRANDOM_X_Y(-winSize.height * 0.25, winSize.height * 0.25);
    //monster.position = ccp(player.spawnPoint.x + player.contentSize.width/2, player.spawnPoint.y + randomOffset);
    monster.position = ccp(player.position.x + player.contentSize.width/2, player.position.y + randomOffset);
    monster.attacking = player.attacking;
    
    [_batchNode addChild:monster];
    [_gameObjects[player.team-1] addObject:monster];
}

- (void)spawnMunchForPlayer:(Player *)player {
    if (player.coins < COST_MUNCH) return;
    player.coins -= COST_MUNCH;
    [[SimpleAudioEngine sharedEngine] playEffect:@"spawn.wav"];
    
    MunchMonster * monster = [[MunchMonster alloc] initWithTeam:player.team layer:self];
    
    CGSize winSize = [CCDirector sharedDirector].winSize;
    float randomOffset = CCRANDOM_X_Y(-winSize.height * 0.25, winSize.height * 0.25);
    //monster.position = ccp(player.spawnPoint.x + player.contentSize.width/2, player.spawnPoint.y + randomOffset);
    monster.position = ccp(player.position.x + player.contentSize.width/2, player.position.y + randomOffset);
    monster.attacking = player.attacking;
    
    [_batchNode addChild:monster];
    [_gameObjects[player.team-1] addObject:monster];
}

- (void)quirkButtonTapped:(id)sender {    
    [self spawnQuirkForPlayer:_humanPlayer];
}

- (void)zapButtonTapped:(id)sender {
    [self spawnZapForPlayer:_humanPlayer];
}

- (void)munchButtonTapped:(id)sender {
    [self spawnMunchForPlayer:_humanPlayer];
}

- (void)draw {
    for (NSMutableArray * teamObjects in _gameObjects) {
        for (GameObject * gameObject in teamObjects) {
            [gameObject draw];
        }
    }
    [_aiPlayer draw];
    [_humanPlayer draw];
}

- (void)update:(ccTime)delta {
    
    double time = CACurrentMediaTime();
    
    if (_gameOver) return;
    
    // Check win/lose
    if (_humanPlayer.curHp <= 0) {
        [self showRestartMenu:NO];
    }
    if (_aiPlayer.curHp <= 0) {
        [self showRestartMenu:YES];
    }
    
    // Add coins if necessary
    NSArray * players = @[_humanPlayer, _aiPlayer];
    for (Player * player in players) {
        if (time - player.lastCoinDrop > 1.5) {
            player.lastCoinDrop = time;
            player.coins += 5;
        }
    }
    
    // Set labels
    [_coin1Label setString:[NSString stringWithFormat:@"%d", _humanPlayer.coins]];
    [_coin2Label setString:[NSString stringWithFormat:@"%d", _aiPlayer.coins]];
    
    // Calculate values for AI
    _humanQuirkValue = 0;
    _humanZapValue = 0;
    _humanMunchValue = 0;
    NSArray * humanGameObjects = _gameObjects[0];
    for (GameObject * gameObject in humanGameObjects) {
        if ([gameObject isKindOfClass:[QuirkMonster class]]) {
            _humanQuirkValue += COST_QUIRK;
        } else if ([gameObject isKindOfClass:[ZapMonster class]]) {
            _humanZapValue += COST_ZAP;
        } else if ([gameObject isKindOfClass:[MunchMonster class]]) {
            _humanMunchValue += COST_MUNCH;
        }
    }
    _humanTotalValue = _humanQuirkValue + _humanZapValue + _humanMunchValue;
    
    _aiQuirkValue = 0;
    _aiZapValue = 0;
    _aiMunchValue = 0;
    NSArray * aiGameObjects = _gameObjects[1];
    for (GameObject * gameObject in aiGameObjects) {
        if ([gameObject isKindOfClass:[QuirkMonster class]]) {
            _aiQuirkValue += COST_QUIRK;
        } else if ([gameObject isKindOfClass:[ZapMonster class]]) {
            _aiZapValue += COST_ZAP;
        } else if ([gameObject isKindOfClass:[MunchMonster class]]) {
            _aiMunchValue += COST_MUNCH;
        }
    }
    _aiTotalValue = _aiQuirkValue + _aiZapValue + _aiMunchValue;
            
}

# pragma mark Helper methods

- (int)oppositeTeam:(int)team {
    if (team == 1) {
        return 2;
    } else {
        return 1;
    }
}

- (Player *)playerForTeam:(int)team {
    if (team == 1) {
        return _humanPlayer;
    } else {
        return _aiPlayer;
    }
}

- (NSArray *)alliesOfTeam:(int)team {
    return _gameObjects[team-1];
}

- (NSArray *)enemiesOfTeam:(int)team {
    int oppositeTeam = [self oppositeTeam:team];
    return _gameObjects[oppositeTeam-1];
}

- (NSArray *)enemiesWithinRange:(float)range ofPlayer:(Player *)player {
    NSMutableArray * retval = [NSMutableArray array];;
    NSArray * enemies = [self enemiesOfTeam:player.team];
    for (GameObject * enemy in enemies) {
        float distance = ccpDistance(enemy.position, player.position);
        if (distance < range) {
            [retval addObject:enemy];
        }
    }
    return retval;
}

- (GameObject *)closestEnemyToGameObject:(GameObject *)gameObject {
    
    NSArray * enemies = [self enemiesOfTeam:gameObject.team];
    
    GameObject * closestEnemy = nil;
    float closestEnemyDistance = -1;
    for (GameObject * enemy in enemies) {
        float distance = ccpDistance(gameObject.position, enemy.position);
        if (distance < closestEnemyDistance || closestEnemyDistance == -1) {
            closestEnemy = enemy;
            closestEnemyDistance = distance;
        }
    }
    
    return closestEnemy;
}

- (void)removeGameObject:(GameObject *)gameObject {
    [_gameObjects[gameObject.team-1] removeObject:gameObject];
    [gameObject removeFromParentAndCleanup:YES];
}

- (Laser *)createLaserForTeam:(int)team {
    Laser * laser = [[Laser alloc] initWithTeam:team layer:self];
    [_batchNode addChild:laser];
    return laser;
}

@end
